# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def initialize_calibration(paths_and_names, calibration_settings):
    """ initialize_calibration initialize parameter for ome and mtom calibration.

        The input list of lists "paths_and_names" contains the following values:
            *  [0][0] string: 'path_to_python_scripts',          [0][1] string: 'the system path to python scripts'
            *  [1][0] string: 'path_of_working_directory',       [1][1] string: 'the system path of working directory'
            *  [2][0] string: 'path_of_working_directory_rce',   [2][1] string: 'the system path of working directory for rce'
            *  [3][0] string: 'path_of_origin_tool_directory',   [3][1] string: 'the system path to the root directory of UNICADO repositories'
            *  [4][0] string: 'current_workflow_name',           [4][1] string: 'workflow name of the current workflow execution'
            *  [5][0] string: 'path_to_aircraft_projects',       [5][1] string: 'the system path to the repository of aircraft design projects'
            *  [6][0] string: 'path_to_aircraft_design_tools',   [6][1] string: 'the system path to the repository of aircraft design tools'
            *  [7][0] string: 'path_to_software_tools',          [7][1] string: 'the system path to the repository of software tools'
            *  [8][0] string: 'path_to_json_files',              [8][1] string: 'the path to module json files in .rce system folder'
            *  [9][0] string: 'aircraft_exchange_directory',     [9][1] string: 'name of the directory, witch contains the current aircraft project'
            * [10][0] string: 'aircraft_exchange_file',         [10][1] string: 'name of the current used aircraft exchange file'
            * [11][0] string: 'aircraft_project',               [11][1] string: 'name of the aircraft project, which ist currently designed'
            * [12][0] string: 'path_to_project',                [12][1] string: 'the system path to current project folder'
            * [13][0] string: 'aircraft_type',                  [13][1] string: 'name of aircraft type'

        The input list of lists "calibration_settings" contains the following values:
            *  [0][0] string: 'free_variable_OME_calibration',                   [0][1] int:   'switch for free variable OME calibration'
            *  [1][0] string: 'free_variable_MTOM_calibration',                  [1][1] int:   'switch for free variable MTOM calibration'
            *  [2][0] string: 'calibration_target_OME',                          [2][1] float: 'target value for OME calibration'
            *  [3][0] string: 'calibration_target_mtom',                         [3][1] float: 'target value for MTOM calibration'
            *  [4][0] string: 'reset_fuselage_mass_at_start',                    [4][1] int:   'switch to reset free OME calibration values at start'
            *  [5][0] string: 'reset_viscous_drag_counts_at_start',              [5][1] int:   'switch to reset aerodynamic MTOM calibration value at start'
            *  [6][0] string: 'reset_engine_fuel_flow_factor_at_start',          [6][1] int:   'switch to reset engine MTOM calibration value at start'
            *  [7][0] string: 'number_of_iterations_before_mtom_calibration',    [7][1] int:   'maximum number of OME calibration steps before performing a MTOM calibration'
            *  [8][0] string: 'max_iterations_before_exit_OME_calibration',      [8][1] int:   'maximum number of OME calibration steps before exit the calibration'
            *  [9][0] string: 'trunc_Error_factor',                              [9][1] int:   'value for termination criterion -> as factor regarding convergence criteria'
            * [10][0] string: 'damp_MTOM_calibration_lever',                    [10][1] int:   'switch for damping MTOM calibration lever arm'
            * [11][0] string: 'x_rel_max_for_MTOM_damp',                        [11][1] float: 'value of relative MTOM calibration lever arm'
            * [12][0] string: 'damp_OME_calibration_lever',                     [12][1] int:   'Switch for damping OME calibration lever arm'
            * [13][0] string: 'x_rel_max_for_OME_damp',                         [13][1] float: 'value of relative OME calibration lever arm'

        The output list of lists "calibration_settings" contains the following values:
            *  [0][0] string: 'free_variable_OME_calibration',                   [0][1] int:   'switch for free variable OME calibration'
            *  [1][0] string: 'free_variable_MTOM_calibration',                  [1][1] int:   'switch for free variable MTOM calibration'
            *  [2][0] string: 'calibration_target_OME',                          [2][1] float: 'target value for OME calibration'
            *  [3][0] string: 'calibration_target_mtom',                         [3][1] float: 'target value for MTOM calibration'
            *  [4][0] string: 'reset_fuselage_mass_at_start',                    [4][1] int:   'switch to reset free OME calibration values at start'
            *  [5][0] string: 'reset_viscous_drag_counts_at_start',              [5][1] int:   'switch to reset aerodynamic MTOM calibration value at start'
            *  [6][0] string: 'reset_engine_fuel_flow_factor_at_start',          [6][1] int:   'switch to reset engine MTOM calibration value at start'
            *  [7][0] string: 'number_of_iterations_before_mtom_calibration',    [7][1] int:   'maximum number of OME calibration steps before performing a MTOM calibration'
            *  [8][0] string: 'max_iterations_before_exit_OME_calibration',      [8][1] int:   'maximum number of OME calibration steps before exit the calibration'
            *  [9][0] string: 'trunc_Error_factor',                              [9][1] int:   'value for termination criterion -> as factor regarding convergence criteria'
            * [10][0] string: 'damp_MTOM_calibration_lever',                    [10][1] int:   'switch for damping MTOM calibration lever arm'
            * [11][0] string: 'x_rel_max_for_MTOM_damp',                        [11][1] float: 'value of relative MTOM calibration lever arm'
            * [12][0] string: 'damp_OME_calibration_lever',                     [12][1] int:   'Switch for damping OME calibration lever arm'
            * [13][0] string: 'x_rel_max_for_OME_damp',                         [13][1] float: 'value of relative OME calibration lever arm'
            * [14][0] string: 'residual_iteration_value',                       [14][1] float: 'initial value of ome calibration residual'
            * [15][0] string: 'residual_iteration_value_to_target',             [15][1] float: 'initial value of ome calibration residual to target'
            * [16][0] string: 'calibration_factor_temp',                        [16][1] float: 'temporary value of ome calibration factor'
            * [17][0] string: 'calibration_factor_flag',                        [17][1] float: 'current value of ome calibration flag'
            * [18][0] string: 'calibration_factor_low'                          [18][1] float: 'value of lower calibration factor'
            * [19][0] string: 'calibration_factor_high'                         [19][1] float: 'value of higher calibration factor'
            * [20][0] string: 'calibration_factor_act',                         [20][1] float: 'value of actual calibration factor'
            * [21][0] string: 'calibration_factor_last',                        [21][1] float: 'value of last calibration factor'
            * [22][0] string: 'calibration_value_flag',                         [22][1] bool:  'current status of calibration value flag'
            * [23][0] string: 'calibration_value_low',                          [23][1] float: 'lower calibration value'
            * [24][0] string: 'calibration_value_high'                          [24][1] float: 'higher calibration value'
            * [25][0] string: 'calibration_value_act'                           [25][1] float: 'actual calibration value'
            * [26][0] string: 'calibration_value_last',                         [26][1] float: 'last calibration value'
            * [27][0] string: 'aux_interpolation_flag',                         [27][1] bool:  'status of auxiliary interpolation flag'
            * [28][0] string: 'calibration_value_over_target_exist',            [28][1] bool:  'status of calibration value over target value'
            * [29][0] string: 'calibration_value_under_target_exist',           [29][1] bool:  'status of calibration value under target value'
            * [30][0] string: 'calibration_value_over_target_update_exist',     [30][1] bool:  'status of calibration value update over target value'
            * [31][0] string: 'calibration_value_under_target_update_exist',    [31][1] bool:  'status of calibration value update under target value'
            * [32][0] string: 'number_reverse_effects',                         [32][1] int:   'number of reverse effects which are performed during calibration'
            * [33][0] string: 'number_of_final_runs',                           [33][1] int:   'number of final runs performed during calibration'
            * [34][0] string: 'x_relative'                                      [34][1] float: 'value of relative lever arm in x direction'
            * [35][0] string: 'max_delta_calibration_factor'                    [35][1] float: 'value of maximum allowed delta for calibration factor'
            * [36][0] string: 'calibrate_ome_flag',                             [36][1] bool:  'flag for ome calibration'
            * [37][0] string: 'calibrate_mtom_flag',                            [37][1] bool:  'flag for mtom calibration'
            * [38][0] string: 'final_fuselage_mass_factor',                     [38][1] float: 'value of final fuselage mass factor'
            * [39][0] string: 'current_value',                                  [39][1] float: 'value of current operating mass empty'
            * [40][0] string: 'last_value',                                     [40][1] float: 'value of last operating mass empty'
            * [41][0] string: 'iteration_flag'                                  [41][1] int:   'number of the status of calibration successful'

        :param: paths_and_names: input list of lists
        :param: calibration_settings: input list of lists
        :return: calibration_settings: output list of lists
    """

    ''' imports for python '''
    from calibration.reset_free_calibration_variables import reset_free_calibration_variables
    from calibration.set_settings_for_ome_calibration import set_settings_for_ome_calibration
    from calibration.save_calibration_results_to_file import save_calibration_results_to_file
    from sub_function.write_to_log_file import write_to_log_file

    ''' create list for log-file string '''
    log_file_list = ['********************************************** initialize calibration '
                     '**********************************************']

    ''' read data for script execution '''
    current_workflow_name = ([item for item in paths_and_names if 'current_workflow_name' in item])[-1][-1]
    path_of_working_directory_rce\
        = ([item for item in paths_and_names if 'path_of_working_directory_rce' in item])[-1][-1]

    calibration_check_flag = int(([item for item in \
                                    calibration_settings if 'free_variable_OME_calibration' in item])[-1][-1] > 0 or \
                                 ([item for item in \
                                    calibration_settings if 'free_variable_MTOM_calibration' in item])[-1][-1] > 0)

    # check if calibration mode active -> if true: -> initialize calibration mode operations
    if calibration_check_flag:
        # reset free variables for OME and MTOM calibration
        log_file_list = reset_free_calibration_variables(paths_and_names, calibration_settings, log_file_list)

        # call function to set settings for ome calibration or set final values if OME calibration is skipped
        calibration_settings, log_file_list = set_settings_for_ome_calibration(paths_and_names, calibration_settings,
                                                                               log_file_list)

        # save initial calibration values to csv file
        save_calibration_results_to_file(paths_and_names, 0, 0.000000, 'OME')

    ''' write log-file to system '''
    log_file_list.append('******************************************** end initialize calibration '
                         '********************************************')
    log_file_list.append('')

    write_to_log_file(path_of_working_directory_rce, current_workflow_name, log_file_list)

    return calibration_settings
