# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

"""
A temporary API module just as an example if the sensitivity analysis is necesary.
(As a theory, the results of the sensitivity analysis is dependent on the output and the ranges of the input.)

Author: Daigo Maruyama
Technical contact: d.maruyama@tu-braunschweig.de
"""

### standard libraries (includig numpy, scipy) ###
import numpy as np

### the modules in this package ###
import gaussianProcess, doe         # stand-alone modules in the package
import userinput

### a special library for Sobol Indices - SALib ###
#from SALib.sample import saltelli
#from SALib.analyze import sobol


class globalSensitivity:
    def __init__(self, X, Y, multiStart=10):
        self.surrogate = gaussianProcess.GP.MLE(X, Y, regularization=False, multiStart=multiStart)

    def ard(self):
        sensitivity = self.surrogate.automatic_relevance_determination()
        # surrogate update based on acqisitionFunction (with method="variance") can be inserted here to enhance the quality of the surrogate model efficiently.
        return sensitivity

    def sobolIndices(self):
        NotImplementedError("can be added when necessary.")
        """
        a process using self.surrogate and the module SALib
        """
#        return sensitivity


if __name__ == "__main__":
    """
    The easiest way in the current API is:

    just execute main.py
    Then, do:
    M = len(gbounds)
    for i in range(1+M):
        print(formulation.surrogate[i].automatic_relevance_determination())
    """

    """
    Need to generate Data: (X, Y):
    """

    # if you already have a dataset:
#    X = np.loadtxt("DoE")              # bounds = [ [600, 650], [0.3, 0.34] ]
#    Y = np.loadtxt("mission_energy_data").reshape(-1, 1)

    # Otherwise, generate a dataset by yourself:
    N = 75
    X = doe.QMC.Sobol(N, bounds=[ [-2., 2.], [-1., 3.] ])[:]
    Y = np.empty((N, 1), dtype=np.float64)
    delX = np.empty(N, dtype=str)
    for i in range(N):
        Y[i], delX[i] = userinput.functions2D(X[i])                        # your function evaluation (e.g. extracted from userinput.py)
    if len(delX) > 0:
        X = np.array([i for j, i in enumerate(X) if j not in delX])

    print("X =", X)
    print("Y =", Y)

    ingredients = globalSensitivity(X, Y, multiStart=1)

    sensitivityARD = ingredients.ard()
    print(sensitivityARD)

#    sensitivitySobol = ingredients.sobolIndices()
#    print(sensitivitySobol)
