# UNICADO - UNIversity Conceptual Aircraft Design and Optimization
#
# Copyright (C) 2025 UNICADO consortium
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# Description:
# This file is part of UNICADO.

def recursive_data_remove(path_to_data_to_remove):
    """ recursive_data_delete Changes the access rights of all files inside the given directory to read, write and execute, afterwards the given directory is deleted.

        The input string "path_to_data_to_delete" contains the system path to directory to be recursive deleted.

        :param: path_to_data_to_remove: input string
        :return: none
    """

    ''' imports for python '''
    import os
    import shutil
    import stat
    import time

    ''' loop across all directories and files inside the given directory to change access rights '''
    for root, dirs, files in os.walk(path_to_data_to_remove):
        for directory in dirs:
            os.chmod(os.path.join(root, directory), stat.S_IRWXU)
        for file in files:
            os.chmod(os.path.join(root, file), stat.S_IRWXU)

    ''' recursive remove of given directory '''
    while_count = 0
    while True:
        try:
            shutil.rmtree(path_to_data_to_remove)
            break
        except OSError:
            if while_count == 10:
                print('Warning: Recursive directory remove failed. Delete operation is skipped.')
                break

            while_count += 1
            print('Warning: Try to remove directory recursively.')
            print('Retry ' + str(while_count) + ' of 10!')
            time.sleep(3)
